/**************************************************************************\
  class Mesh
  ==========

  PointGroup-type class which contains a triangle mesh
\**************************************************************************/

#ifndef _MESH_H_
#define _MESH_H_

#include "PointGrp.h"
#include "Parser.h"

class Mesh : public PointGroup
{
public:
    Mesh(bool inputIsExtraCompPCM=false);
    ~Mesh();

    // Inherited methods:
    const std::string& Name();

    // Specific methods:
    inline unsigned TriangleCount()
    { return triangles.size()-smoothTriangles; }
    inline unsigned SmoothTriangleCount() { return smoothTriangles; }


//======================================================================
private:
    // Inherited methods:
    bool ReadPovray(InputStream&);
    bool ReadPCM(InputStream&);
    bool WritePovray(std::ostream&);
    bool WritePovray2(std::ostream&);
    bool WritePCM(std::ostream&);

    bool WritePCM3(std::ostream&);

    // Specific methods:
    struct Triangle
    {
        PointPtr p1, p2, p3, n1, n2, n3;
        bool smooth;

        Triangle();
    };

    std::vector<Triangle> triangles;
    unsigned smoothTriangles, degenerateTriangles;

    bool ReadTriangle(Parser&);
    bool ReadSmoothTriangle(Parser&);
    void SeparatePointsFromNormals();
    void Report();
    void Optimize();
    unsigned RemoveUnusedPoints();
    void CalculateExtraCompression();
    unsigned Adjacent(Triangle*,unsigned,unsigned,unsigned,unsigned);
    bool isSmoothAdjacent(Triangle*,unsigned,unsigned,unsigned);
    void NonAdjacentIndex(Triangle*,unsigned,unsigned&,unsigned&);

    std::vector<unsigned> extraCompData;
    bool inputIsExtraCompPCM;
};

#endif

