/**************************************************************************\
  class Object
  ============
\**************************************************************************/

#include "Object.h"

#include <cstdio>
#include <cstdlib>
using namespace std;

Object::~Object() { }

// Reading and writing
//=========================================================================
bool Object::Read(InputStream& is, FileType type)
{
    switch(type)
    {
      case POVRAY: return ReadPovray(is);
      case POVRAY2: return ReadPovray2(is);
      case PCM: return ReadPCM(is);
    }
    return true;
}
bool Object::Write(ostream& os, FileType type)
{
    switch(type)
    {
      case POVRAY: return WritePovray(os);
      case POVRAY2: return WritePovray2(os);
      case PCM: return WritePCM(os);
    }
    return true;
}

bool Object::ReadPovray(InputStream&) { return false; }
bool Object::ReadPovray2(InputStream&) { return false; }
bool Object::ReadPCM(InputStream&) { return false; }
bool Object::WritePovray(ostream&) { return false; }
bool Object::WritePovray2(ostream&) { return false; }
bool Object::WritePCM(ostream&) { return false; }

// Apply transformations
//=========================================================================
bool Object::ApplyTransformations(
    const vector<string>& transformations)
{
    double f[7];

    for(unsigned i=0; i<transformations.size(); i++)
    {
        switch(transformations[i][1])
        { case 's': if(transformations[i][2]=='w')
                    {
                        if(!SwapCoords(transformations[i][5],
                                       transformations[i][6]))
                          return false;
                    }
                    else
                    {
                        if(!Scale(atof(transformations[i].substr(2).c_str())))
                          return false;
                    }
                    break;
          case 'c': if(!Center((transformations[i].size()==3 &&
                               transformations[i][2]=='b')))
                      return false;
                    break;
          case 'b': if(transformations[i].size()>2)
                    {
                        if(transformations[i].size()==3 &&
                           (transformations[i][2]=='-' ||
                            transformations[i][2]=='+'))
                        {
                            if(!PrintBoundaries())
                              return false;
                        }
                        else
                        {
                            if(!Scale2Bound(
                                atof(transformations[i].substr(2).c_str())))
                              return false;
                        }
                    }
                    else
                    {
                        if(!Scale2Bound(2))
                          return false;
                    }
                    break;
          case 'r': sscanf(transformations[i].substr(2).c_str(),
                           "%lg,%lg,%lg,%lg,%lg,%lg,%lg",
                           &f[0],&f[1],&f[2],&f[3],&f[4],&f[5],&f[6]);
                    if(!Rotate(f[0],Point(f[1],f[2],f[3]),
                               Point(f[4],f[5],f[6])))
                      return false;
                    break;
          case 't': sscanf(transformations[i].substr(2).c_str(),
                           "%lg,%lg,%lg",&f[0],&f[1],&f[2]);
                    if(!Translate(Point(f[0],f[1],f[2])))
                      return false;
                    break;
        }
    }

    if(Options::Round()) RoundValues();
    return true;
}

// Default transformations:
//=========================================================================
bool Object::Center(bool bottom, bool verbose, const Limits* l)
{
    Point coords;

    if(l==0) l = &currentLimits;

    coords.x( -(l->max.x()+l->min.x())/2 );
    if(bottom) coords.y( -l->min.y() );
    else coords.y( -(l->max.y()+l->min.y())/2 );
    coords.z( -(l->max.z()+l->min.z())/2 );

    return Translate(coords, verbose);
}

bool Object::Scale2Bound(double size, bool verbose, const Limits* l)
{
    if(l==0) l = &currentLimits;

    if(!Center(false, verbose, l)) return false;

    double scl;
    Point dim(l->max.x()-l->min.x(),
              l->max.y()-l->min.y(),
              l->max.z()-l->min.z());
    if(dim.x()>dim.y())
    { if(dim.x()>dim.z()) scl=dim.x();
      else scl=dim.z();
    }
    else
    { if(dim.y()>dim.z()) scl=dim.y();
      else scl=dim.z();
    }
    scl=size/scl;

    return Scale(scl, verbose);
}

bool Object::PrintBoundaries(const char* caption, const Limits* l)
{
    if(l==0) l = &currentLimits;
    Options::PrintMsg("%s:\n  Min = <"DOUBLE", "DOUBLE", "DOUBLE">\n"
                      "  Max = <"DOUBLE", "DOUBLE", "DOUBLE">\n",
                      caption,
                      l->min.x(), l->min.y(), l->min.z(),
                      l->max.x(), l->max.y(), l->max.z());
    return true;
}

